@@ -4,28 +4,28 @@
 #
 # A command line tool for managing machine learning models.
 #
-# Copyright 2018 (c) Graham.Williams@togaware.com All rights reserved. 
+# Copyright 2018 (c) Graham.Williams@togaware.com All rights reserved.
 #
 # This file is part of mlhub.
 #
 # MIT License
 #
-# Permission is hereby granted, free of charge, to any person obtaining a copy 
-# of this software and associated documentation files (the ""Software""), to deal 
-# in the Software without restriction, including without limitation the rights 
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
-# copies of the Software, and to permit persons to whom the Software is 
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the ""Software""), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
 # furnished to do so, subject to the following conditions:
 #
-# The above copyright notice and this permission notice shall be included in 
+# The above copyright notice and this permission notice shall be included in
 # all copies or substantial portions of the Software.
 #
-# THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
+# THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
 import distro
@@ -96,7 +96,7 @@ def list_available(args):
         utils.print_meta_line(entry)
 
     # Suggest next step.
-    
+
     if not args.quiet:
         utils.print_next_step('available')
         if not os.path.exists(utils.get_init_dir()):
@@ -135,15 +135,15 @@ def list_installed(args):
     if args.name_only:
         print('\n'.join(models))
         return
-        
+
     # Report on how many models we found installed.
-        
+
     mcnt = len(models)
     plural = "s" if mcnt != 1 else ""
     print("Found {} model{} installed{}".format(mcnt, plural, msg))
 
     # Report on each of the installed models.
-        
+
     if mcnt > 0:
         print("")
 
@@ -174,7 +174,7 @@ def list_installed(args):
         print(utils.get_command_suggestion('remove'))
 
     # Suggest next step.
-    
+
     if not args.quiet:
         if mcnt > 0:
             utils.print_next_step('installed', scenario='exist')
@@ -200,14 +200,17 @@ def install_model(args):
     logger.info('Install a model.')
     logger.debug('args: {}'.format(args))
 
-    model = args.model   # model pkg name
-    location = args.model     # pkg file path or URL
-    version = None       # model pkg version
-    mlhubyaml = None     # MLHUB.yaml path or URL
+    model = args.model     # model pkg name
+    location = args.model  # pkg file path or URL
+    key = args.i           # SSH key
+    version = None         # model pkg version
+    mlhubyaml = None       # MLHUB.yaml path or URL
+    repo_obj = None        # RepoTypeURL object for related URL interpretation
+    maybe_private = False  # Maybe private repo
 
     # Obtain the model URL if not a local file.
 
-    if not utils.is_archive(model) and not utils.is_url(model) and '/' not in model:
+    if not utils.is_archive_file(model) and not utils.is_url(model) and '/' not in model:
 
         # Model package name, which can be found in mlhub repo.
         # Like:
@@ -230,35 +233,57 @@ def install_model(args):
 
         utils.update_model_completion({e['meta']['name'] for e in meta_list})
 
-    if not utils.is_archive(location):
+    if not utils.is_archive_file(location):
 
-        # Model from GitHub.
-        # Like:
-        #     $ ml install mlhubber/audit
-        #     $ ml install mlhubber/audit:doc/MLHUB.yaml
-        #     $ ml install https://github.com/mlhubber/audit/...
+        # Model from a repo such as GitHub, GitLab, Bitbucket etc.
+        #
+        # Possible options are:
+        #   $ ml install mlhubber/audit            # latest commit on the master branch of GitHub repo mlhubber/audit
+        #   $ ml install mlhubber/audit@dev        # latest commit on the dev branch of GitHub repo mlhubber/audit
+        #   $ ml install mlhubber/audit@0001ea4    # commit 0001ea4 of mlhubber/audit
+        #   $ ml install mlhubber/audit:doc/MLHUB.yaml            # latest commit on master, but a specified YAML file
+        #   $ ml install https://github.com/mlhubber/audit/...    # Arbitrary GitHub link address
+        #
+        #   $ ml install github:mlhubber/audit    # GitHub repo, the same as ml install mlhubber/audit
+        #
+        #   $ ml install gitlab:mlhubber/audit@2fe89kh:doc/MLHUB.yaml    # GitLab repo
+        #   $ ml install https://https://gitlab.com/mlhubber/audit/...   # GitLab repo
+        #
+        #   $ ml install bitbucket:mlhubber/audit                        # BitBucket repo
+        #   $ ml install https://bitbucket.org/mlhubber/audit/...        # BitBucket repo
 
-        mlhubyaml = utils.get_pkgyaml_github_url(location)  # URL to MLHUB.yaml
-        location = utils.get_githubrepo_zip_url(location)
-        logger.debug("location: {}".format(location))
-        logger.debug("mlhubyaml: {}".format(mlhubyaml))
+        repo_obj = utils.RepoTypeURL.get_repo_obj(location)
+        try:
+            mlhubyaml = repo_obj.get_pkg_yaml_url()
+            location = repo_obj.compose_repo_zip_url()
+            logger.debug("location: {}".format(location))
+            logger.debug("mlhubyaml: {}".format(mlhubyaml))
+        except utils.DescriptionYAMLNotFoundException:  # Maybe private repo
+            maybe_private = True
+            pass
 
     # Determine the path of downloaded/existing model package file
 
     pkgfile = None
-    if utils.is_archive(location):
+    if maybe_private:  # Maybe private repo
+        pkgfile = repo_obj.repo
+    elif utils.is_archive_file(location):
         pkgfile = os.path.basename(location)  # pkg file name
     elif utils.is_url(location):
         pkgfile = utils.get_url_filename(location)
 
     # Query archive type if not available from file name per se.
 
-    while pkgfile is None or not utils.is_archive(pkgfile):
-        print("The file type cannot be determined.\n"
-              "Please give it a file name with explicit valid archive extension: ", end='')
-        pkgfile = input()
+    if not maybe_private:
+        while pkgfile is None or not utils.is_archive_file(pkgfile):
+            print("The file type cannot be determined.\n"
+                  "Please give it a file name with explicit valid archive extension: ", end='')
+            pkgfile = input()
 
-    uncompressdir = pkgfile[:pkgfile.rfind('.')]  # Dir Where pkg file is extracted
+    if maybe_private:
+        uncompressdir = pkgfile
+    else:
+        uncompressdir = pkgfile[:pkgfile.rfind('.')]  # Dir Where pkg file is extracted
 
     # Installation.
 
@@ -267,7 +292,9 @@ def install_model(args):
 
         # Determine the local path of the model package
 
-        if utils.is_url(location):
+        if maybe_private:
+            local = None
+        elif utils.is_url(location):
             local = os.path.join(mlhubtmpdir, pkgfile)  # downloaded
         else:
             local = location  # local file path
@@ -281,7 +308,9 @@ def install_model(args):
 
                 model, version = utils.interpret_mlm_name(pkgfile)
 
-            elif not utils.is_github_url(location):  # Get MLHUB.yaml inside the archive file.
+            elif not repo_obj:
+
+                # Get MLHUB.yaml inside the archive file.
 
                 if utils.is_url(location):  # Download the package file because it is not from GitHub.
                     utils.download_model_pkg(location, local, pkgfile, args.quiet)
@@ -292,6 +321,21 @@ def install_model(args):
                 utils.unpack_with_promote(local, uncompressdir, valid_name=pkgfile)
                 mlhubyaml = utils.get_available_pkgyaml(uncompressdir)  # Path to MLHUB.yaml
 
+            elif maybe_private:
+
+                identity_env = "GIT_SSH_COMMAND='ssh -i {}' ".format(key) if key else ''
+                command = "cd {}; {}git clone {}; cd {}; git checkout {}".format(
+                    mlhubtmpdir, identity_env, repo_obj.get_ssh_clone_url(), repo_obj.repo, repo_obj.ref)
+                proc = subprocess.Popen(command, shell=True, stderr=subprocess.PIPE)
+                output, errors = proc.communicate()
+                if proc.returncode != 0:
+                    raise utils.InstallFailedException(errors.decode("utf-8"))
+
+                if repo_obj.path:
+                    mlhubyaml = os.path.join(uncompressdir, repo_obj.path)
+                else:
+                    mlhubyaml = utils.get_available_pkgyaml(uncompressdir)  # Path to MLHUB.yaml
+
             if mlhubyaml is not None:  # Get version number from MLHUB.yaml
                 entry = utils.read_mlhubyaml(mlhubyaml)
                 meta = entry["meta"]
@@ -390,7 +434,7 @@ def install_model(args):
                                         model,
                                         downloadir=uncompressdir,
                                         yes=True)
-            except utils.ModePkgInstallationFileNotFoundException as e:
+            except utils.ModelPkgInstallationFileNotFoundException:
                 if os.path.exists(install_path):
                     shutil.rmtree(install_path)
 
@@ -398,7 +442,7 @@ def install_model(args):
 
         else:
             # Otherwise, put all files under package dir.
-            # **Note** Here we must make sure <instal_path> does not exist.
+            # **Note** Here we must make sure <install_path> does not exist.
             # Otherwise, <unzipdir> will be inside <install_path>
             shutil.move(uncompressdir, install_path)
 
@@ -408,8 +452,8 @@ def install_model(args):
 
         # Update working dir if any.
 
-        if args.workding_dir is not None:
-            utils.update_working_dir(model, args.workding_dir)
+        if args.working_dir is not None:
+            utils.update_working_dir(model, args.working_dir)
 
         if not args.quiet:
 
@@ -434,13 +478,13 @@ def download_model(args):
     # download.sh script. Which ever (maybe either), if it is present
     # then this command is available and will download the required
     # file, perhaps from the actual source of the model.
-    
+
     model = args.model
-   
+
     # Check that the model is installed.
 
     utils.check_model_installed(model)
-    
+
     if not args.quiet:
         utils.print_next_step('download', model=model)
 
@@ -535,14 +579,14 @@ def list_model_commands(args):
     # Check that the model is installed.
 
     utils.check_model_installed(model)
-    
+
     entry = utils.load_description(model)
     commands = entry['commands']
 
     if args.name_only:
         print('\n'.join(list(commands)))
         return
-    
+
     msg = "The '{}' model "
     meta = entry['meta']
     if 'title' not in meta:
@@ -564,7 +608,7 @@ def list_model_commands(args):
     utils.update_command_completion(set(commands))
 
     # Suggest next step.
-    
+
     if not args.quiet:
         utils.print_next_step('commands', description=entry, model=model)
 
@@ -594,7 +638,7 @@ def configure_model(args):
     #
     # 4 Assume model packager provides a configure.R script. This is a
     #   override and no other configuration happens if this is
-    #   supplied. Alternatively this is viewed as a cop-out prividing
+    #   supplied. Alternatively this is viewed as a cop-out providing
     #   no support from mlhub for the model packager. The preference
     #   would be to use the dependencies: tag to list the required R
     #   or python packages.
@@ -609,7 +653,7 @@ def configure_model(args):
     #     if(length(new.packages)) install.packages(install)
     #   else if language: == "python":
     #     packages = dependencies:
-    #     cat pacakges > requirements.txt
+    #     cat packages > requirements.txt
     #     pip install -r requirements.txt
     #
 
@@ -629,10 +673,7 @@ def configure_model(args):
             proc = subprocess.Popen(command, shell=True, cwd=path, stderr=subprocess.PIPE)
             output, errors = proc.communicate()
             if proc.returncode != 0:
-                errors = errors.decode("utf-8")
-                print("\nAn error was encountered:\n")
-                print(errors)
-                raise utils.ConfigureFailedException()
+                raise utils.ConfigureFailedException(errors.decode("utf-8"))
 
         return
 
@@ -649,7 +690,7 @@ def configure_model(args):
     # Setup.
 
     pkg_dir = utils.get_package_dir(model)
-   
+
     # Check if the model package is installed.
 
     utils.check_model_installed(model)
@@ -724,7 +765,7 @@ def configure_model(args):
             # ----- Files -----
 
             elif 'files'.startswith(category):
-                utils.install_file_deps(deplist, model, yes=YES)
+                utils.install_file_deps(deplist, model, key=args.i, yes=YES)
 
     # Run additional configure script if any.
 
@@ -743,11 +784,11 @@ def configure_model(args):
 
     # Update working dir if any.
 
-    if args.workding_dir is not None:
-        utils.update_working_dir(model, args.workding_dir)
-            
+    if args.working_dir is not None:
+        utils.update_working_dir(model, args.working_dir)
+
     # Suggest next step.
-    
+
     if not args.quiet:
         utils.print_next_step('configure', model=model)
 
@@ -767,10 +808,10 @@ def dispatch(args):
 
     # Get working dir if any.
 
-    if args.workding_dir is not None:
-        utils.update_working_dir(model, args.workding_dir)
-        if args.workding_dir == '':
-            args.workding_dir = None
+    if args.working_dir is not None:
+        utils.update_working_dir(model, args.working_dir)
+        if args.working_dir == '':
+            args.working_dir = None
     else:
         args.working_dir = utils.get_working_dir(model)
 
@@ -781,7 +822,7 @@ def dispatch(args):
     # Check that the model is installed and has commands.
 
     utils.check_model_installed(model)
-    
+
     entry = utils.load_description(model)
 
     if 'commands' not in entry or len(entry['commands']) == 0:
@@ -813,20 +854,20 @@ or else connect to the server's desktop using a local X server like X2Go.
     lang = meta["languages"]
 
     # Deal with malformed 'languages' field
-    
+
     lang_opts = {"python": "py", "R": "R"}
     for k in list(lang_opts):
         if lang in k:
             lang = lang_opts[k]
             break
-        
+
     # Obtain the specified script file.
-    
+
     script = cmd + "." + lang
 
     logger = logging.getLogger(__name__)
     logger.debug("Execute the script: " + os.path.join(path, script))
-     
+
     if cmd not in list(entry['commands']) or not os.path.exists(os.path.join(path, script)):
         raise utils.CommandNotFoundException(cmd, model)
 
@@ -838,31 +879,9 @@ or else connect to the server's desktop using a local X server like X2Go.
 
     # Change working dir if needed
 
-    if args.workding_dir is not None:
+    if args.working_dir is not None:
         script = os.path.join(path, script)
-        path = args.workding_dir
-
-    # Handle python environment
-
-    python_pkg_bin = None
-    python_pkg_path = None
-    if script.endswith('py'):
-        python_pkg_base = os.path.sep.join([utils.get_package_dir(model), '.python'])
-        python_pkg_path = python_pkg_base + site.USER_SITE
-        python_pkg_bin = python_pkg_base + site.USER_BASE + '/bin'
-
-        # TODO: Make sure to document:
-        #     $ sudo apt-get install -y python3-pip
-        #     $ /usr/bin/pip3 install mlhub
-        #   Since in DSVM, the default pip is conda's pip, so if we stick to
-        #   use system's command, then the installation of MLHub itself should
-        #   be completed via system's pip, otherwise, MLHub will not work.
-
-        if sys.executable != SYS_PYTHON_CMD:
-            python_pkg_path = python_pkg_base + site.getsitepackages()[0]
-            python_pkg_bin = python_pkg_base + site.PREFIXES[0]+ '/bin'
-            if utils.get_sys_python_pkg_usage(model):
-                utils.print_on_stderr(MSG_INCOMPATIBLE_PYTHON_ENV, model)
+        path = args.working_dir
 
     # _MLHUB_CMD_CWD: a environment variable indicates current working
     #                 directory where command `ml xxx` is invoked.
@@ -874,20 +893,20 @@ or else connect to the server's desktop using a local X server like X2Go.
 
     env_var = "export _MLHUB_CMD_CWD='{}'; ".format(os.getcwd())
     env_var += "export _MLHUB_MODEL_NAME='{}'; ".format(model)
-    env_var += 'export _MLHUB_PYTHON_EXE="{}"; '.format(sys.executable)
-    env_var += "export PYTHONPATH='{}'; ".format(python_pkg_path) if python_pkg_path else ""
-    env_var += "export PATH=\"{}:$PATH\"; ".format(python_pkg_bin) if python_pkg_bin else ""
+    env_var += 'export _MLHUB_PYTHON_EXE="{}"; '.format(sys.executable) if not conda_env_name else ""
+    if not conda_env_name and script.endswith('py'):  # Handle python environment
+        env_var += utils.get_py_pkg_path_env(model)
 
-    command = "{}{} {} {}".format(env_var, interpreter, script, param)
+    command = "{}{} {} {}".format(env_var, interpreter if not conda_env_name else "python", script, param)
 
     # Run script inside conda environment if specified
 
     if conda_env_name is not None:
-        command = '{} -c "source activate {}; {}"'.format(BASH_CMD, conda_env_name, command)
+        command = '{} -ic "conda activate {}; {}"'.format(BASH_CMD, conda_env_name, command)
 
     logger.debug("(cd " + path + "; " + command + ")")
 
-    proc = subprocess.Popen(command, shell=True, cwd=path, stderr=subprocess.PIPE)
+    proc = subprocess.Popen(command, shell=True, cwd=path)
     output, errors = proc.communicate()
     missing_r_dep = False
     if proc.returncode != 0:
@@ -897,7 +916,7 @@ or else connect to the server's desktop using a local X server like X2Go.
 
         dep_required = re.compile(r"ModuleNotFoundError: No module named '(.*)'").search(errors)
 
-        # Check if R dependency unsatisified
+        # Check if R dependency unsatisfied
 
         if dep_required is None:
             dep_required = re.compile(r"there is no package called ‘(.*)’").search(errors)
@@ -919,9 +938,10 @@ or else connect to the server's desktop using a local X server like X2Go.
             print(errors)
 
     else:
-        # Suggest next step
+        # Suggest next step - in the context of the command line view
+        # of the tool, let's not provide a next step for now. 20190528
 
-        if not args.quiet:
+        if False:  # not args.quiet:
             utils.print_next_step(cmd, description=entry, model=model)
 
 
@@ -957,7 +977,7 @@ def remove_model(args):
     """Remove installed model."""
 
     # TODO: Remove .archive and .config for the model.
-    
+
     model = args.model
 
     # Determine if remove all model pkgs or a certain model pkg.
@@ -984,7 +1004,7 @@ def remove_model(args):
         if os.path.exists(utils.get_package_cache_dir(model)):
             cache = utils.get_package_cache_dir(model)
         msg = "Remove '{}'"
-        
+
         # Check that the model is installed.
 
         utils.check_model_installed(model)
