@@ -4,28 +4,28 @@
 #
 # A command line tool for managing machine learning models.
 #
-# Copyright 2018 (c) Graham.Williams@togaware.com All rights reserved. 
+# Copyright 2018 (c) Graham.Williams@togaware.com All rights reserved.
 #
 # This file is part of mlhub.
 #
 # MIT License
 #
-# Permission is hereby granted, free of charge, to any person obtaining a copy 
-# of this software and associated documentation files (the ""Software""), to deal 
-# in the Software without restriction, including without limitation the rights 
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
-# copies of the Software, and to permit persons to whom the Software is 
+# Permission is hereby granted, free of charge, to any person obtaining a copy
+# of this software and associated documentation files (the ""Software""), to deal
+# in the Software without restriction, including without limitation the rights
+# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+# copies of the Software, and to permit persons to whom the Software is
 # furnished to do so, subject to the following conditions:
 #
-# The above copyright notice and this permission notice shall be included in 
+# The above copyright notice and this permission notice shall be included in
 # all copies or substantial portions of the Software.
 #
-# THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
+# THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
 import collections
@@ -84,7 +84,7 @@ CMD = "ml"                 # The command line tool.
 EXT_MLM = ".mlm"    # Archive filename extension
 EXT_AIPK = ".aipk"  # Backward compatibility
 
-VERSION = "3.4.9"  # DO NOT MODIFY. Managed from ../Makefile.
+VERSION = "3.5.2"  # DO NOT MODIFY. Managed from ../Makefile.
 
 OPTIONS = {
 
@@ -103,7 +103,8 @@ OPTIONS = {
          'action': 'store_true',
         },
     '--quiet':
-        {'help': "reduce noise.",
+        {'alias': ['-q', ],
+         'help': "reduce noise.",
          'action': 'store_true',
         },
     '--init-dir':
@@ -114,7 +115,7 @@ OPTIONS = {
         {'help': "command display name instead of '{}'.".format(CMD),
          'dest': 'mlmetavar',
         },
-    '--workding-dir':
+    '--working-dir':
         {'alias': ['--wd', ],
          'help': "use this as the working dir instead of '{}'/<model>.".format(MLINIT),
         },
@@ -173,7 +174,9 @@ COMMANDS = {
 
     'install':
         {'description': "install a named model, local model file or URL",
-            'argument': {'model': {}},
+            'argument': {'model': {},
+                         '-i': {'help': "SSH key path"},
+                        },
                'usage': "  install    <model>   "
                         "install a named model, local model file or URL",
                 'func': "install_model",
@@ -222,6 +225,7 @@ COMMANDS = {
                          '--yes': {'action': 'store_true',
                                    'help': 'assume "yes" as answer to all prompts',
                                },
+                         '-i': {'help': "SSH key path"},
                         },
                'usage': "  configure [<model>]  "
                         "configure ml or the model's dependencies",
@@ -342,7 +346,7 @@ SYS_PYTHON_PKG_USAGE = "sys_python_pkg_usage"  # Whether system python packages
 # Command binary
 # ------------------------------------------------------------------------
 
-BASH_CMD ='/bin/bash'
+BASH_CMD = '/bin/bash'
 
 R_CMD = '/usr/bin/R'
 RSCRIPT_CMD = '/usr/bin/Rscript'
@@ -355,7 +359,7 @@ SYS_PIP_CMD = '/usr/bin/pip3'
 # Messages
 # ------------------------------------------------------------------------
 
-MSG_INCOMPATIBLE_PYTHON_ENV ="""
+MSG_INCOMPATIBLE_PYTHON_ENV = """
 WARNING: MLHub is not installed in system's site package directory!
 The "{}" MLHub package requires system Python packages, which may not correctly
 be included in the its Python search path.
@@ -367,3 +371,4 @@ followed by re-login.
 Or for advanced users, you could manually `pip install` those required Python
 packages.
 """
+
