@@ -34,6 +34,8 @@ import sys
 import requests
 import termios
 import tty
+import subprocess
+import re
 
 # ----------------------------------------------------------------------
 # Support Package Developers
@@ -66,14 +68,16 @@ def load_key(path):
 
 # Either load key/endpoint from file or ask user and save to file.
 
-def azkey(key_file, service="Cognitive Services"):
+def azkey(key_file, service="Cognitive Services", verbose=True, baseurl=False):
     """The user is asked for an Azure subscription key and endpoint. The
     provided information is saved into a file for future use. The
     contents of that file is the key and endpoint with the endpoint
-    identified as starting with http:
+    identified as starting with http. Some endpoints may include the
+    full cognitive service path and so the baseurl option will strip
+    to just the base name of the URL.
 
     a14d1234abcda4f2f6e9f565df34ef24
-    https://westus2.api.cognitive.microsoft.com/
+    https://westus2.api.cognitive.microsoft.com
 
     """
 
@@ -98,7 +102,7 @@ the file and use this information.
 """.format(service, key_file)
 
     msg_saved = """
-I've saved that information into the file:
+That information has been saved into the file:
 
     {}
 """.format(key_file)
@@ -106,20 +110,31 @@ I've saved that information into the file:
     # Obtain the key/endpoint.
     
     if os.path.isfile(key_file) and os.path.getsize(key_file) > 0:
-        print(msg_found)
+        if verbose: print(msg_found, file=sys.stderr)
         key, endpoint = load_key(key_file)
     else:
-        print(msg_request)
+        print(msg_request, file=sys.stderr)
         
         key      = ask_password(prompt_key)
-        endpoint = input(prompt_endpoint)
+        sys.stderr.write(prompt_endpoint)
+        endpoint = input()
 
         if len(key) > 0 and len(endpoint) > 0:
             ofname = open(key_file, "w")
-            ofname.write("{}\n{}\n".format(key, endpoint))
+            # Use the explicit format in case endpoint has no http prefix.
+            ofname.write("key={}\nendpoint={}\n".format(key, endpoint))
             ofname.close()
-            print(msg_saved)
+            print(msg_saved, file=sys.stderr)
 
+    if baseurl:
+        from urllib.parse import urlsplit
+        splurl = urlsplit(endpoint)
+        endpoint = splurl.scheme + "://" + splurl.netloc
+
+    # Ensure endpoint ends in /
+
+    if endpoint[len(endpoint)-1] != "/": endpoint = endpoint + "/"
+        
     return key, endpoint
 
 # Simple input of password.
@@ -134,8 +149,8 @@ def ask_password(prompt=None):
 
     symbol = "`~!@#$%^&*()-_=+[{]}\\|;:'\",<.>/?"
     if prompt:
-        sys.stdout.write(prompt)
-        sys.stdout.flush()
+        sys.stderr.write(prompt)
+        sys.stderr.flush()
 
     fd = sys.stdin.fileno()
     old_settings = termios.tcgetattr(fd)
@@ -150,17 +165,17 @@ def ask_password(prompt=None):
                 raise KeyboardInterrupt
             if c == '\x7f':  # Backspace.
                 if chars:
-                    sys.stdout.write('\b \b')
-                    sys.stdout.flush()
+                    sys.stderr.write('\b \b')
+                    sys.stderr.flush()
                     del chars[-1]
                 continue
             if c.isalnum() or c in symbol:
-                sys.stdout.write('*')
-                sys.stdout.flush()
+                sys.stderr.write('*')
+                sys.stderr.flush()
                 chars.append(c)
     finally:
         termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
-        sys.stdout.write('\n')
+        sys.stderr.write('\n')
 
     return ''.join(chars)
 
@@ -189,13 +204,37 @@ def azrequest(endpoint, url, subscription_key, request_data):
         raise Exception(response.text)
 
 def mlask(begin="", end=""):
-    print(begin + "Press Enter to continue: ", end=end)
+    sys.stdout.write(begin + "Press Enter to continue: ")
     answer = input()
+    sys.stdout.write(end)
 
 def mlcat(title="", text="", delim="=", begin="", end="\n"):
     sep = delim*len(title) + "\n" if len(title) > 0 else ""
     ttl_sep = "\n" if len(title) > 0 else ""
     print(begin + sep + title + ttl_sep + sep + ttl_sep + text, end=end)
 
-def mlpreview(fname, previewer="display"):
-    os.system(previewer + " " + fname)
+def mlpreview(fname,
+              begin="\n",
+              msg="Close the graphic window using Ctrl-w.\n",
+              previewer="eog"):
+    print(begin + msg)
+    subprocess.Popen([previewer, fname])
+
+# From Simon Zhao's azface package on github.
+
+def is_url(url):
+    """Check if url is a valid URL."""
+
+    urlregex = re.compile(
+        r'^(?:http|ftp)s?://'  # http:// or https://
+        r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'  # domain...
+        r'localhost|'  # localhost...
+        r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # ...or ip
+        r'(?::\d+)?'  # optional port
+        r'(?:/?|[/?]\S+)$', re.IGNORECASE)
+
+    if re.match(urlregex, url) is not None:
+        return True
+    else:
+        return False
+
